# Copyright (C) 2012 Team tiramisu (see README for all contributors)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# The original `Config` design model is unproudly borrowed from 
# the rough gus of pypy: pypy: http://codespeak.net/svn/pypy/dist/pypy/config/
# the whole pypy projet is under MIT licence
# ____________________________________________________________
"enables us to carry out a calculation and return an option's value"
from tiramisu.error import DisabledOptionError, SpecialOwnersError
# ____________________________________________________________
# automatic Option object
special_owners = ['auto', 'fill']
                  
def special_owner_factory(name, owner, value, 
                          callback, callback_params=None, config=None):
    # in case of an 'auto' and a 'fill' without a value, 
    # we have to carry out a calculation
    return calc_factory(name, callback, callback_params, config)

def calc_factory(name, callback, callback_params, config):
    # FIXME we have to know the exact status of the config 
    # not to disrupt it
    # config.freeze()
    if callback_params is None:
        callback_params = {}
    tcparams = {}
    one_is_multi = False
    len_multi = 0
    for key, value in callback_params.items():
        if type(value) == tuple:
            path, check_disabled = value
            try:
                opt_value = getattr(config, path)
                opt = config.unwrap_from_path(path)
            except DisabledOptionError, e:
                if chek_disabled:
                    continue
                raise DisabledOptionError(e)
            is_multi = opt.is_multi()
            if is_multi:
                if opt_value != None:
                    len_value = len(opt_value)
                    if len_multi != 0 and len_multi != len_value:
                        raise SpecialOwnersError('unable to carry out a calculation, '
                        'option values with multi types must have same length for: '
                         + name)
                    len_multi = len_value
                one_is_multi = True
            tcparams[key] = (opt_value, is_multi)
        else:
            tcparams[key] = (value, False)

    if one_is_multi:
        ret = []
        for incr in range(len_multi):
            tcp = {}
            for key, couple in tcparams.items():
                value, ismulti = couple
                if ismulti and value != None:
                    tcp[key] = value[incr]
                else:
                    tcp[key] = value
            ret.append(calculate(name, callback, tcp))
        return ret
    else:
        tcp = {}
        for key, couple in tcparams.items():
            tcp[key] = couple[0]
        return calculate(name, callback, tcp)
    
def calculate(name, callback, tcparams):
    try:
        # XXX not only creole...
        from creole import eosfunc
        return getattr(eosfunc, callback)(**tcparams) 
    except AttributeError, err:
        import traceback
        traceback.print_exc()
        raise SpecialOwnersError("callback: {0} return error {1} for "
                       "option: {2}".format(callback, str(err), name))

